local ADDON_NAME, GHC = ...
-- GHC_EventHandler = {...}
GHC_EventHandler = {}


function GHC_EventHandler:PlayerDataCommHandler(prefix, message, distribution, sender)
    GHC_Core:ParsePlayerData(message, sender)
end


function GHC_EventHandler:GuildBankMoneyCommHandler(prefix, message, distribution, sender)
    -- GHC_Core:Debug("GuildBankMoneyCommHandler", prefix, string.len(message), message, distribution, sender)

    -- TODO Rework this set of 3 functions (Start, Parse, Finish) to be more aptly named since
    -- AceComm handles packetizing large messages automatically
    GHC_Bank:GuildBankStart(message)
end


function GHC_EventHandler:GuildBankDataCommHandler(prefix, message, distribution, sender)
    -- GHC_Core:Debug("GuildBankDataCommHandler", prefix, string.len(message), message, distribution, sender)
    -- TODO Rework this set of 3 functions (Start, Parse, Finish) to be more aptly named since
    -- AceComm handles packetizing large messages automatically
    GHC_Bank:ParseGuildBankData(message, sender)
end


function GHC_EventHandler:GuildBankFinishCommHandler(prefix, message, distribution, sender)
    -- GHC_Core:Debug("GuildBankFinishCommHandler", prefix, string.len(message), message, distribution, sender)
    -- TODO Rework this set of 3 functions (Start, Parse, Finish) to be more aptly named
    GHC_Bank:GuildBankFinish()
end


function GHC_EventHandler:UpdateMeCommHandler(prefix, message, distribution, sender)
    -- GHC_Core:Debug("UpdateMeCommHandler", prefix, string.len(message), message, distribution, sender)
    if GHC_Core.db.global.CurrentVersion ~= message then
        GHC_Core:Print("New version available!")
        GHC_Core.db.global.NewVersion = message
    end
end


--- Default Comm handler
function GHC_EventHandler:OnCommReceived(prefix, message, distribution, sender)
    -- GHC_Core:Debug("OnCommReceived", prefix, string.len(message), message, distribution, sender)
end


-- GetGuildInfo("player") should be available now
function GHC_EventHandler:GUILD_ROSTER_UPDATE(eventName, canRequestRosterUpdate)
end


-- GetNumGuildMembers() is available now
function GHC_EventHandler:GUILD_RANKS_UPDATE(eventName)
    -- Only need to register the Guildspace once
    if GHC_Core:RegisterGuildspace(GHC_Core:GetPlayerGuildInfo()) then
        GHC_Core:UnregisterEvent(eventName)

        GHC_Core:Load()

        local _, player = GHC_Core:UpdatePlayerDataTable()
        local isPlayerEligibleGuildBank = GHC_Bank:IsPlayerEligibleGuildBank(player)

        -- Only register Guild Bank events if you are the Guild Bank Alt
        if player.Role == #GHC.Db.RoleIconTextString then
            if isPlayerEligibleGuildBank then
                GHC_EventHandler:RegisterGuildBankEvents()
            else
                GHC_Core:Printf("WARNING: %q no longer meets the requirements of a Guild Bank alt", player.Name)
                GHC_Core:Printf(
                    "WARNING: The Guild Bank character must be ranked either %q or %q or be listed in the Guild Information window and must have its Character Role set to %q",
                    GuildControlGetRankName(2), GuildControlGetRankName(1),  -- These are 1-indexed
                    "Guild Bank"
                )
                GHC_Core:Print("If you were recently promoted or listed in the Guild Information window as a Guild Bank alt then you must log out and back in")
            end
        end
        GHC_Core:SendMyData()
    end
end


function GHC_EventHandler:PlayerDataChangedHandler()
    --[===[@debug@
    -- GHC_Core:Debug("PlayerDataChangedHandler")
    --@end-debug@]===]
    if GHC_Core.db.global.AutoSendCharacterData then
        GHC_Core:SendMyData()
    end
end

function GHC_EventHandler:SKILL_LINES_CHANGED(eventName)
    -- GHC_Core:Debug(eventName)
end


--- TODO Reset Guildspace and/or register new Guildspace
function GHC_EventHandler:PLAYER_GUILD_UPDATE(eventName)
    GHC_Core:Print("Please logout and back in to fix Guild Helper Classic!")
    local guildName = GHC_Core:GetPlayerGuildInfo()
    if guildName then
    end
end


function GHC_EventHandler:BANKFRAME_OPENED(eventName)
    -- GHC_Core:Print(eventName, GHC_Bank.isBankOpen, "-->", true, GHC_Bank.updateBagTimerId, "-->", nil)
    -- Wait until Bank Frame closes (BANKFRAME_CLOSED) to ScanGuildBank
    GHC_Bank.isBankOpen = true
    -- GHC_Bank:CancelUpdateBagTimer()
end


function GHC_EventHandler:BANKFRAME_CLOSED(eventName)
    -- GHC_Core:Print(eventName, GHC_Bank.isBankOpen)
    -- Ensure we only acknowledge the first BANKFRAME_CLOSED event
    if GHC_Bank.isBankOpen then
        -- GHC_Bank:CancelUpdateBagTimer()
        GHC_Bank:ScanGuildBank()
        GHC_Bank:SendGuildBankData()
    end
    GHC_Bank.isBankOpen = false
end


function GHC_EventHandler:RegisterGuildBankEvents()
    --[===[@debug@
    -- GHC_Core:Debug("RegisterGuildBankEvents", GHC_Bank.areGuildBankEventsRegistered)
    --@end-debug@]===]
    if GHC_Bank.areGuildBankEventsRegistered then return end

    GHC_EventHandler:RegisterEvent("BANKFRAME_OPENED")
    GHC_EventHandler:RegisterEvent("BANKFRAME_CLOSED")
    -- GHC_EventHandler:RegisterEvent("BAG_UPDATE", "UpdateBags")
    -- GHC_EventHandler:RegisterEvent("PLAYER_MONEY", "UpdateBags")
    GHC_Bank.areGuildBankEventsRegistered = true
end


function GHC_EventHandler:UnregisterGuildBankEvents()
    -- GHC_Core:Debug("UnegisterGuildBankEvents", GHC_Bank.areGuildBankEventsRegistered)
    if not GHC_Bank.areGuildBankEventsRegistered then return end
    GHC_Bank.areGuildBankEventsRegistered = false

    -- Cancel any pending updates
    GHC_Bank:CancelUpdateBagTimer()

    GHC_EventHandler:UnregisterEvent("BANKFRAME_OPENED")
    GHC_EventHandler:UnregisterEvent("BANKFRAME_CLOSED")
    -- GHC_EventHandler:UnregisterEvent("BAG_UPDATE")
    -- GHC_EventHandler:UnregisterEvent("PLAYER_MONEY")
end


function GHC_EventHandler:RegisterEvent(eventName, callback)
    if not callback then callback = eventName end
    GHC_Core:RegisterEvent(eventName, function(...) self[callback](self, ...) end)
end

function GHC_EventHandler:UnregisterEvent(...) GHC_Core:UnregisterEvent(...) end


function GHC_EventHandler:RegisterBucketEvent(events, timeout, callback)
    GHC_Core:RegisterBucketEvent(events, timeout, function(...) self[callback](self, ...) end)
end


function GHC_EventHandler:RegisterComm(prefix, callback)
    GHC_Core:RegisterComm(prefix, function(...) self[callback](self, ...) end)
end
